%% Stochastic Simulation of the Lotka-Volterra Reactions 
%
% Generates and stochastically simulates the following model:
%
% * Reaction 1: y1 -> 2 y1, with rate constant, c1 = 1.
% * Reaction 2: y1 + y2 -> 2 y2, with rate constant, c2 = 0.005.
% * Reaction 3: y2 -> null, with rate constant, c3 = 0.6.
% * Initial conditions: y1=y2=20,
%
% These reactions can be interpreted as a simple predator-prey model if 
% one considers that the prey (y1) population increases in the presence 
% of food (x) (Reaction 1), that the predator (y2) population increases 
% as they eat prey (y1) animals (Reaction 2), and that predators (y2) 
% die of natural causes (Reaction 3).
%

clear all; clc; close all;

yinit(1,:) = [50 2.^[3:8]];
yinit(2,:) = 2*yinit(1,:);

% Create the Lotka-Volterra Model
model = sbiomodel('Lotka-Volterra Model');


% Add Reaction 1 to the Model Object
r1 = addreaction(model,'y1 -> 2 y1');

% Set the Kinetic Law for Reaction 1.
kl1 = addkineticlaw(r1, 'MassAction');

% Add rate constant parameter, c1, to reaction with value = 1
p1 = addparameter(kl1, 'c1', 'Value', 1);

kl1.ParameterVariableNames = {'c1'};


% Set initial amounts for species in Reaction 1
set(r1.Reactants(1),'InitialAmount',20);          %y1

% Add Reaction 2 to the Model Object
r2 = addreaction(model,'y1 + y2 -> 2 y2');

% Set the kinetic law for Reaction 2.
kl2 = addkineticlaw(r2, 'MassAction');

% Add rate constant parameter, c2, to kinetic law with value = 0.005
p2 = addparameter(kl2, 'c2', 'Value', 0.005);

kl2.ParameterVariableNames = {'c2'};

% Add Reaction 3 to the Model Object
r3 = addreaction(model,'y2 -> null');
% Set initial amounts for new species in Reaction 3
set(r3.Reactants(1),'InitialAmount',20);          %y2

% Set the kinetic law for Reaction 3.
kl3 = addkineticlaw(r3, 'MassAction');

% Add rate constant parameter, c3, to reaction with value = 0.6
p3 = addparameter(kl3, 'c3', 'Value', 0.6);

kl3.ParameterVariableNames = {'c3'};


%% One realisation (SSA run)

cs = getconfigset(model);
set(cs, 'SolverType', 'ssa', 'StopTime', 30);
cs.SolverOptions.LogDecimation = 10;

cs.SolverOptions.RandomState = [];
[t_ssa,n_ssa] = sbiosimulate(model);
save lv_stoch_20x20 t_ssa n_ssa

if exist('yinit')
    for i=1:length(yinit)
        clear t_ssa n_ssa
        set(r1.Reactants(1),'InitialAmount',yinit(1,i));
        set(r3.Reactants(1),'InitialAmount',yinit(2,i));
        cs.SolverOptions.RandomState = [];
        [t_ssa,n_ssa] = sbiosimulate(model);
        save(['lv_stoch_',num2str(yinit(1,i)), 'x', ...
                      num2str(yinit(2,i))], 't_ssa', 'n_ssa')
    end
end