% Examples for Phase Plane Analysis.
% Left-click in the phase plane to select initial conditions for a time
% plot. This opens a new window for the time history. Right-click once to
% undo the last time plot. Any other other keyboard key will terminate the
% plotting.

% Examples from Chapter 7 (Boyce Diprima)

%% Section 7.5, Example 1
% Saddle point in the origin (unstable fixed point).
% Eigenvalues:  (real, distinct)  
%     3.0000         0
%          0   -1.0000
% Eigenvectors:   
%     0.4472   -0.4472
%     0.8944    0.8944
A = [1 1; 4 1];
phasport(A, [-3 3]);
% saveas(gcf,'bd7_5ex1_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 1.5 -4 4])
% saveas(gcf,'bd7_5ex1_th.eps', 'psc2')

%% Section 7.5, Example 2
% Node in the origin. All solutions approach origin. Nodes are
% asymptotically stable if the eigenvalues are negative and unstable if the
% eigenvalues are positive.
% Eigenvalues:  (real, distinct)
%    -4.0000         0
%          0   -1.0000
% Eigenvectors:  
%    -0.8165   -0.5774
%     0.5774   -0.8165
A = [-3 sqrt(2);sqrt(2) -2];
phasport(A, [-3 3]);
% saveas(gcf,'bd7_5ex2_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 1.5 -2 2])
% saveas(gcf,'bd7_5ex2_th.eps', 'psc2')

%% Section 7.6, Example 1
% Spiral point in the origin. Complex eigenvalues. 
% Inward spiral - asymptotically stable. For a system whose eigenvalues have a 
% positive real part, the trajectories are similar but the direction is away from the origin.  
% Eigenvalues:    
%   -0.5000 + 1.0000i        0          
%    0            -0.5000 - 1.0000i 
% Eigenvectors:  
%    0.7071             0.7071          
%    0 + 0.7071i        0 - 0.7071i
A = [-.5 1;-1 -.5];
phasport(A, [-3 3]);
% saveas(gcf,'bd7_6ex1_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 5 -2 2])
% saveas(gcf,'bd7_6ex1_th.eps', 'psc2')

%% Section 7.8, Example 2
% Improper node in the origin. Repeated eigenvalues. If the eigenvalues are
% negative, then the trajectories are similar, but are traversed in the
% inward direction. An improper node is asymptotically stable or unstable,
% depending on whether the eigenvalues are negative or positive.
% Eigenvalues:   
%     2.0000         0
%          0    2.0000  
% Eigenvectors:   
%    -0.7071   -0.7071
%     0.7071    0.7071
A = [1 -1;1 3];
phasport(A, [-3 3]);
% saveas(gcf,'bd7_8ex2_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 3 -3 3])
% saveas(gcf,'bd7_8ex2_th.eps', 'psc2')

%% Section 9.2, Example 1
% Hyperbola
% Eigenvalues:   
%     -1     0
%      0     1
% Eigenvectors:  
%    -0.7071    0.7071
%     0.7071    0.7071
A = [0 1;1 0];
phasport(A, [-2 2]);
% saveas(gcf,'bd9_2ex1_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 3 -2 2])
% saveas(gcf,'bd9_2ex1_th.eps', 'psc2')

%% Section 9.2, Example 2
% Saddle point at t->infinity. Saddle point at (2,2). Center at (-2,2)
odefun = @(t,x) [4 - 2*x(2,:); 12 - 3*x(1,:).^2];
phasport(odefun, [-4.5 4.5 -5 9]);
% saveas(gcf,'bd9_2ex2_pp.eps', 'psc2')
timeplots(odefun, [0 3 -4 8])
% saveas(gcf,'bd9_2ex2_th.eps', 'psc2')

%% Section 9.3, Example 4
% Pendulum. The trajectories that enter the saddle points separate the
% phase plane into regions. Such a trajectory is called a seperatix. Each
% region contains exactly one of the asymptotically stable spiral points.
% The set of all initial points from which trajectories approach a given
% asymptotically stable critical point is called the basin of attraction or
% the region of asymptotic stability for that critical point. In contrast
% to linear systems is that in the nonlinear case not necessarily all
% trajectories end in the critical point.
odefun = @(t,x) [x(2,:); -9*sin(x(1,:)) - .2*x(2,:)];
phasport(odefun, [-9 9 -12 12]);
% saveas(gcf,'bd9_3ex4_pp.eps', 'psc2')
timeplots(odefun, [0 3 -9 9])
% saveas(gcf,'bd9_3ex4_th.eps', 'psc2')

%% Section 9.4, Example 1
% Competing species. Critical point (0,0) - all species die. Saddle point (1,0) - x
% survives and y dies. Saddle point (0,0.75) - y survives but x not. (0.5,
% 0.59 - coexistence.
odefun = @(t,x) [x(1,:).*(1 - sum(x)); x(2,:).*(.75 - x(2,:) - .5*x(1,:))];
phasport(odefun, [0 1.5]);
% saveas(gcf,'bd9_4ex1_pp.eps', 'psc2')
timeplots(odefun, [0 10 0 1.5])
% saveas(gcf,'bd9_4ex1_th.eps', 'psc2')

%% Section 9.5, Example 1
% Predator-Prey system. Origin is an unstable saddle point.
odefun = @(t,x) [x(1,:).*(1 - .5*x(2,:)); x(2,:).*(-.75 + .25*x(1,:))];
phasport(odefun, [0 8 0 5]);
% saveas(gcf,'bd9_5ex1_pp.eps', 'psc2')
[t,y] = ode45(odefun, [0 30], [2;1]);
figure, plot(t,y(:,1),'-b',t,y(:,2),':r');
preprint('time','x1, x2',[],{'prey','predator'})
% saveas(gcf,'bd9_5ex1_th.eps', 'psc2')

%% Section 9.7, Example 1
% Limit cycle, which is asymptotically stable. (0,0) is the only critical point. The system is almost linear in the
% neighbourhood of the origin. 
odefun = @(t,x) [x(2,:);-x(1,:)] + x.*(1 - x([1;1],:).^2 - x([2;2],:).^2); 
phasport(odefun, [-2 2]);
% saveas(gcf,'bd9_7ex1_pp.eps', 'psc2')
timeplots(odefun, [0 10 -2 2])
% saveas(gcf,'bd9_7ex1_th.eps', 'psc2')

%% Section 9.7, Example 2
% Van der Pol equations. 
mu = .2;
odefun = @(t,x) [x(2,:); -x(1,:) + mu*(1 - x(1,:).^2).*x(2,:)]; 
phasport(odefun, [-2.5 2.5]);
% saveas(gcf,'bd9_7ex2_pp.eps', 'psc2')
timeplots(odefun, [0 50 -2.5 2.5])
% saveas(gcf,'bd9_7ex2_th.eps', 'psc2')

%-----More Linear examples-----------------------------------

%% Hyperbola
% dx/dt=2x + y dy/dt=2x-y . Steady-state in origin. Jacobian=[2 1;2 -1]
% Eigenvalues:    
%     2.5616         0
%          0   -1.5616
% Eigenvectors:  
%     0.8719   -0.2703
%     0.4896    0.9628
A = [2 1; 2 -1];
phasport(A, [-2 2]);
% saveas(gcf,'hyperb_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 5 -2 2])
% saveas(gcf,'hyperb_th.eps', 'psc2')

%% Hyperbola through origin (uncoupled equations)
% Eigenvalues:  
%  
%     2.0000         0
%          0   -2.0000
% Eigenvectors:  
%  
%     0.4472   -0.4472
%     0.8944    0.8944
A = [0 1; 4 0];
phasport(A, [-10 10]);
%saveas(gcf,'hyperb2_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 1 -10 10])
%saveas(gcf,'hyperb2_th.eps', 'psc2')

%% Circle
%Eigenvalues:   
%        0 + 1.0000i        0          
%        0                  0 - 1.0000i
%Eigenvectors:   
%   0.7071             0.7071          
%   0 + 0.7071i        0 - 0.7071i
A = [0 1;-1 0];
phasport(A, [-10 10]);
% saveas(gcf,'circle_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 10 -10 10])
% saveas(gcf,'circle_th.eps', 'psc2')

%% example3
% Eigenvalues:   
%     -3     0
%      0    -1
% Eigenvectors:  
%     0.7071    0.7071
%    -0.7071    0.7071
A = [-2 1;1 -2];
phasport(A, [-2 2]);
% saveas(gcf,'ex3_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 5 -2 2])
% saveas(gcf,'ex3_th.eps', 'psc2')

%% example4
% Eigenvalues:   
%     -3     0
%      0     1 
% Eigenvectors:  
%     0.7071   -0.7071
%     0.7071    0.7071
A = [-1 -2;-2 -1];
phasport(A, [-2 2]);
% saveas(gcf,'ex4_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 5 -2 2])
% saveas(gcf,'ex4_th.eps', 'psc2')

%% example5
% Star.
% Eigenvalues:  
%      1     0
%      0     1
% Eigenvectors:  
%      1     0
%      0     1
A = eye(2);
phasport(A, [-10 10]);
% saveas(gcf,'ex5_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 5 -2 2])
% saveas(gcf,'ex5_th.eps', 'psc2')

%% example6
% Eigenvalues:  
%      1     0
%      0     1
% Eigenvectors:  
%          0    0.0000
%     1.0000   -1.0000
A = [1 0;2 1];
phasport(A, [-10 10]);
% saveas(gcf,'ex6_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 5 -2 2])
% saveas(gcf,'ex6_th.eps', 'psc2')

%% example7
% Spiral inward.
% Eigenvalues:  
%   -2.0000 + 3.0000i        0          
%         0            -2.0000 - 3.0000i
% Eigenvectors:  
%    0.7071             0.7071          
%         0 + 0.7071i        0 - 0.7071i
A = [-2 3;-3 -2];
phasport(A, [-10 10]);
% saveas(gcf,'ex7_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 2 -2 2])
% saveas(gcf,'ex7_th.eps', 'psc2')

%% example8
% Spiral outward.
% Eigenvalues:   
%    4.0000 + 3.0000i        0          
%         0             4.0000 - 3.0000i
% Eigenvectors:  
%    0.7071             0.7071          
%         0 - 0.7071i        0 + 0.7071i
A = [4 -3;3 4];
phasport(A, [-10 10]);
saveas(gcf,'ex8_pp.eps', 'psc2')
timeplots(@(t,x) A*x, [0 2 -1000 1000])
saveas(gcf,'ex8_th.eps', 'psc2')

%-----More Nonlinear examples-----------------------------------

%% bilinear
% Steady states: trivial at (0,0), nontrivial at (-1,-3). Jacobian matrix =
% [y x+1;y+3 x]
% alim =  [-5 5];
% jac = jacobian(sym('[y*(x + 1) ; x*(y + 3)]'));
% disp('Jacobian:  '); disp(' '); pretty(jac);
% xss = [0;0];
% A = subs(jac,{'x','y'},{xss(1),xss(2)});
% phasport(A,alim);
% saveas(gcf,'bilinear_pp1.eps', 'psc2')
% figure
% xss = [-1;-3];
% A = subs(jac,{'x','y'},{xss(1),xss(2)});
% linear = @(x) [A(1,1)*x(1,:) + A(1,2)*x(2,:); A(2,1)*x(1,:) + A(2,2)*x(2,:)];
% linodefun = @(t,x) linear([x(1,:) - xss(1); x(2,:) - xss(2)]);
% phasport(linodefun, alim);
% saveas(gcf,'bilinear_pp2.eps', 'psc2')
% figure
odefun = @(t,x) [x(2,:).*(x(1,:) + 1); x(1,:).*(x(2,:) + 3)];
phasport(odefun, [-5 5]);
% saveas(gcf,'bilinear_pp.eps', 'psc2')
timeplots(odefun, [0 5 -5 5])
% saveas(gcf,'bilinear_th.eps', 'psc2')

%% Bioreactor with Monod kinetics
mumax = .53; km = .12; Y = .4; sf = 4;
mu = @(s) mumax*s./(km + s);
D = .8*mu(sf);
odefun = @(t,x) [(mu(x(2,:))-D).*x(1,:); (sf - x(2,:))*D - mu(x(2,:)).*x(1,:)/Y];
phasport(odefun, [0 5]);
% saveas(gcf,'bioreactor__pp.eps', 'psc2')
timeplots(odefun, [0 10 0 5])
% saveas(gcf,'bioreactor__th.eps', 'psc2')
% syms x y
% mu = mumax*y./(km + y);
% jac = jacobian([(mu - D)*x; (sf - y)*D - mu*x/Y]);
% disp('Jacobian:  '); disp(' '); pretty(jac);