function hFig = prepfigure(varargin)
%PREPFIGURE   prepares figure for plotting
%
%   PREPFIGURE, the same as FIGURE, creates a new figure window
%
%   PREPFIGURE([W H]) addittionally sets the 'Position' and 'PaperSize'
%   so that both the figure window and the output paper is WxH in centimeters.
%
%   PREPFIGURE([W H], UNITSTR) uses the units UNITSTR which can be one of
%   the strings {'cm', 'centimeter', 'inches', 'pt', 'points', 'normalized'}
%
%   PREPFIGURE(PROP1, VAL1, PROP2, VAL2,...) sets the specified properties 
%   for the width ('w'), height ('h') and unit ('u') in a parameter/value 
%   pair manner. The two methods must not be mixed, however. Additional
%   valid figure properties can also be passed.
%
%   PREPFIGURE(FIG, ...) sets the specified properties to the figure with
%   handle FIG
%
% Mukhtar Ullah
% mukhtar.ullah@informatik.uni-rostock.de
% First created: Feb 5, 2008
% Last mofified: Sep 27, 2010

[fig, args] = figurecheck(varargin{:});

if isempty(fig)
    fig = figure();
    defaultFigSize = [8 6];
else
    figure(fig);
    defaultFigSize = [];
end

p1 = inputParser;
p2 = p1.createCopy();

p1.addOptional('figsize', defaultFigSize, @(x) isnumeric(x) && numel(x)==2);

validUnits = {'cm', 'centimeter', 'inches', 'pt', 'points'};

p1.addOptional('units', 'centimeter',  @(x) ischar(x) && any(strncmpi(x,validUnits,2)));

validateRange = @(x) validateattributes(x, {'numeric'}, {'scalar', 'positive'});

p2.addParamValue('w', [], validateRange);
p2.addParamValue('h', [], validateRange);
p2.addParamValue('u', 'centimeter', @ischar);

try
    p1.KeepUnmatched = true;
    p1.parse(args{:});
    figsize = p1.Results.figsize;
    units = validatestring(p1.Results.units, validUnits);
    newArgs = p1.Unmatched;
    fCell = mixed_input_check('w', 'h', 'u');
catch e1;
    try
        p2.KeepUnmatched = true;
        p2.parse(args{:});
        w = p2.Results.w;
        h = p2.Results.h;
        units = validatestring(p2.Results.u, validUnits);
        newArgs = p2.Unmatched;
        fCell = mixed_input_check('figsize', 'units');        
        if isempty(h) && ~isempty(w)
            h = (3/4)*w;
        elseif isempty(w) && ~isempty(h)
            w = (4/3)*h;
        end        
        if ~isempty(w) || ~isempty(h)
            figsize = [w h];
        end
    catch e2;
        exception = MException( ...
            'prepfigure:arglist', ...
            'Error in input argument list');
        exception = addCause(exception, e1);
        exception = addCause(exception, e2);
        throw(exception)
    end
end


switch units
    case 'cm'
        units = 'centimeter';
    case 'pt'
        units = 'points';
end

if ~isempty(figsize)
    oldunits = get(fig, {'Units', 'PaperUnits'});
    set(fig, 'Units', units, 'PaperUnits', units);
    figpos = get(fig, 'Position');
    figpos(3:4) = figsize;
    set(fig, 'Position', figpos, ...
        'Units', oldunits{1}, ...
        'PaperUnits', oldunits{2});
end


if ~isempty(fCell)    
    set(gcf,fCell',struct2cell(newArgs)')    
end

if nargout==1
    hFig = fig;
end

    function fCell = mixed_input_check(varargin)
        fCell = fieldnames(newArgs);
        assert(~any(ismember(varargin,fCell)), ...
            'Mixed use of optional arguments and parameter/value pairs not allowed')
    end

end


