function rm2sf(textobj)
% 
% RM2SF changes the typeface of non-math text with latex interpreter from
%  roman (serif) to sans-serif. This is needed because the latex interpreter
%  typesets both math and text in roman irrespective of the font
%  specification. Therefore RM2SF places each non-math part of the text
%  inside \textsf{}.
% 
%  RM2SF replaces, by \textsf{STR}, each non-math part STR of all text 
%  objects in the current figure whose interpreter is set to latex.
% 
%  RM2SF(FIG) applies to text objects in figure with handle FIG.
% 
%  RM2SF(AX) applies to text objects in axes with handle AX.
% 
%  RM2SF(TX) applies to text objects with handles specified in vector TX.
% 
% Example:
% 
% tx(1) = text(0.2,0.8, 'square of \$x\$ is $x^2$');
% tx(2) = text(0.3,0.6, {'sine of $\theta$'; 'is $\sin \theta$'});
% tx(3) = text(0.2,0.4, 'cube of $x$ is $x^3$');
% tx(4) = xlabel('time $t$ here');
% tx(5) = ylabel('concentration {\it C(t)} here', 'FontSize', 20);
% set(tx(1:4), 'FontSize', 20 , 'Interpreter', 'latex')
% rm2sf(tx([1 2 4]))
% 
% Author: Dr. Ing. Mukhtar Ullah
% Created: 18 Oct, 2010
% Last Modified: 20 Oct, 2010

if nargin==0
    textobj = get(0, 'CurrentFigure');
end

assert( ~isempty(textobj) && all(ishandle(textobj)) && ...
    any(ismember(get(textobj,'Type'),{'figure', 'axes', 'text'})), ...
    'Missing or Invalid text/axes/figure object')

htext = findall(textobj, 'Type', 'text', ...
    '-and', 'Interpreter', 'latex', '-not', 'String', '');
     
for h=htext(:)'
    textstr = get(h,'string');
    if ischar(textstr) % single line text
        textstr = textsf(textstr);
    else % multiline text
        i = cellfun(@isempty,textstr);
        textstr(i) = cellfun(@textsf, textstr(i), 'UniformOutput', false);
    end
    set(h, 'string', textstr);
end

function strout = textsf(strin)
if iscellstr(strin) % multiline text e.g. in legend
    strout = cellfun(@textsf, strin, 'UniformOutput', false);
    return
end
% single line text
mathsubstr = '\s*(?<!\\)\${1,2}[^\$]*(?<!\\)\${1,2}\s*'; % math
textsfsubstr = '\s*\\textsf(?:\{.*\})?\s*'; % text already inside \textsf{}
matchExpr = [mathsubstr '|' textsfsubstr];
[matchstr, splitstr] = regexp(strin, matchExpr, 'match', 'split');
i = ~strcmp('',splitstr);
splitstr(i) = cellfun( ...
    @(x) ['\textsf{' x '}'], splitstr(i), 'UniformOutput', false );
joinedstr = [splitstr; [matchstr {''}]];
strout = [joinedstr{:}];
